"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_names_1 = require("./-private/plugin-names");
const plugin_configuration_1 = require("./-private/plugin-configuration");
/**
 * Locates the existing configuration, if any, for a given plugin.
 *
 * @param target An `EmberApp` or `Addon` instance whose configuration should be checked
 * @param plugin The name of the plugin to be located
 */
function findPlugin(target, plugin) {
    let plugins = plugin_configuration_1.getPluginsArray(target);
    let index = plugin_configuration_1.findPluginIndex(target, plugin);
    return plugins[index];
}
exports.findPlugin = findPlugin;
/**
 * Indicates whether the given plugin is already present in the target's configuration.
 *
 * @param target An `EmberApp` or `Addon` instance whose configuration should be checked
 * @param plugin The name of the plugin to be located
 */
function hasPlugin(target, plugin) {
    return !!findPlugin(target, plugin);
}
exports.hasPlugin = hasPlugin;
/**
 * Add a plugin to the Babel configuration for the given target.
 *
 * @param target An `EmberApp` or `Addon` instance for which the plugin should be set up
 * @param plugin Configuration for the Babel plugin to add
 * @param options Optional constraints around where the plugin should appear in the array
 */
function addPlugin(target, plugin, options = {}) {
    let earliest = Math.max(...findPluginIndices(target, options.after)) + 1;
    let latest = Math.min(...findPluginIndices(target, options.before));
    if (earliest > latest) {
        throw new Error(`Unable to satisfy placement constraints for Babel plugin ${plugin_names_1.resolvePluginName(plugin)}`);
    }
    let plugins = plugin_configuration_1.getPluginsArray(target);
    let targetIndex = Number.isFinite(latest) ? latest : Number.isFinite(earliest) ? earliest : plugins.length;
    plugins.splice(targetIndex, 0, plugin);
}
exports.addPlugin = addPlugin;
function findPluginIndices(target, plugins = []) {
    let pluginIndices = plugins.map(name => (name ? plugin_configuration_1.findPluginIndex(target, name) : -1));
    return pluginIndices.filter(index => index >= 0);
}
